@@ -1,9 +1,16 @@
-from cosymlib.molecule import Molecule, Geometry, ElectronicStructure
+from cosymlib.molecule import Molecule
+from cosymlib.molecule.geometry import Geometry
+from cosymlib.molecule.electronic_structure import ElectronicStructure
 from cosymlib.file_io.tools import extract_geometries
-from cosymlib.tools import atomic_number_to_element
+from cosymlib.tools import atomic_number_to_element, element_to_atomic_number
+from cosymlib.file_io import errors
 
 import numpy as np
-import os, sys, re
+import os, re
+import warnings
+
+
+comment_line = ('#', '!', '$')
 
 
 def _non_blank_lines(f):
@@ -13,6 +20,23 @@ def _non_blank_lines(f):
             yield line
 
 
+def no_name(file_name, n_title_structure):
+    if len(os.path.splitext(os.path.basename(file_name))[0]) > 8:
+        return os.path.splitext(os.path.basename(file_name))[0][:8] + '_' + str(n_title_structure)
+    else:
+        return os.path.splitext(os.path.basename(file_name))[0] + '_' + str(n_title_structure)
+
+
+def check_geometries_vertices(geometries, file_name):
+    n_atoms = geometries[0].get_n_atoms()
+    for idg, geometry in enumerate(geometries):
+        if geometry.get_n_atoms() != n_atoms:
+            warnings.warn('Structure {} with name {} of file {}\n '
+                          'Number of vertices does not match with other structures\n'.format(idg + 1, geometry.name,
+                                                                                             file_name),
+                          errors.DifferentVerticesWarning)
+
+
 # Read INPUT files
 def read_generic_structure_file(input_name, read_multiple=False):
     # print('Reading file {}...'.format(os.path.basename(input_name)))
@@ -42,34 +66,58 @@ def get_geometry_from_file_xyz(file_name, read_multiple=False):
     """
     input_molecule = [[], []]
     geometries = []
+    n_atoms = None
+    n_atoms_line = 1
+    no_title_structures = 1
     with open(file_name, mode='r') as lines:
-        lines.readline()
-        name = lines.readline()
-        if name.strip():
-            name = name.split()[0]
-        for line in lines:
-            if '$' in line or '#' in line:
+        for idl, line in enumerate(lines):
+            if line.lstrip().startswith(comment_line):
                 pass
+            elif line.strip() == '':
+                warnings.warn('Line {} is empty in xyz file'.format(idl + 1), errors.EmptyLineWarning)
             else:
                 try:
                     float(line.split()[1])
                     input_molecule[0].append(line.split()[0])
                     input_molecule[1].append(line.split()[1:])
-                except (ValueError, IndexError):
+                except (IndexError, ValueError):
                     if input_molecule[0]:
-                        geometries.append(Geometry(symbols=input_molecule[0],
-                                                  positions=input_molecule[1],
-                                                  name=name))
-                    input_molecule = [[], []]
-                    name = line.split()[0]
+                        if len(input_molecule[0]) != n_atoms:
+                            warnings.warn('Number of atoms in line {} and number '
+                                          'of atoms provided are not equal'.format(n_atoms_line),
+                                          errors.MissingAtomWarning)
+                        if name == '':
+                            name = no_name(file_name, no_title_structures)
+                            no_title_structures += 1
 
-        molecule = Geometry(symbols=input_molecule[0],
-                                  positions=input_molecule[1],
-                                  name=name)
-        if read_multiple:
-            geometries.append(molecule)
-        else:
-            return molecule
+                        geometries.append(Geometry(symbols=input_molecule[0],
+                                                   positions=input_molecule[1],
+                                                   name=name))
+                        n_atoms = None
+
+                    name = ''
+                    input_molecule = [[], []]
+                    if n_atoms is None:
+                        n_atoms = int(line.split()[0])
+                        n_atoms_line = idl + 1
+                    else:
+                        name = line.split()[0]
+
+        if len(input_molecule[0]) != n_atoms:
+            warnings.warn('Number of atoms in line {} and number '
+                          'of atoms provided are not equal'.format(n_atoms_line),
+                          errors.MissingAtomWarning)
+        if name == '':
+            name = no_name(file_name, no_title_structures)
+
+        geometries.append(Geometry(symbols=input_molecule[0],
+                                   positions=input_molecule[1],
+                                   name=name))
+        if not read_multiple:
+            return geometries[0]
+
+    # Check if all geometries are of the same vertices
+    check_geometries_vertices(geometries, file_name)
 
     return geometries
 
@@ -84,12 +132,12 @@ def get_geometry_from_file_cor(file_name, read_multiple=False):
     input_molecule = [[], []]
     geometries = []
     with open(file_name, mode='r') as lines:
-        line = lines.readline()
-        name = line.replace('**FRAG**', '')[:-1]
-        name = name.replace(' ', '')
-        for line in lines:
-            if '$' in line or '#' in line:
+
+        for idl, line in enumerate(lines):
+            if line.lstrip().startswith(comment_line):
                 pass
+            elif line.strip() == '':
+                warnings.warn('Line {} is empty'.format(idl + 1), errors.EmptyLineWarning)
             else:
                 try:
                     float(line.split()[1])
@@ -102,7 +150,7 @@ def get_geometry_from_file_cor(file_name, read_multiple=False):
                     else:
                         float(line.split()[2])
                 except (ValueError, IndexError):
-                    if input_molecule:
+                    if input_molecule[0]:
                         geometries.append(Geometry(symbols=input_molecule[0],
                                                    positions=input_molecule[1],
                                                    name=name))
@@ -114,6 +162,10 @@ def get_geometry_from_file_cor(file_name, read_multiple=False):
                                    name=name))
         if not read_multiple:
             return geometries[0]
+
+    # Check if all geometries are of the same vertices
+    check_geometries_vertices(geometries, file_name)
+
     return geometries
 
 
@@ -122,7 +174,7 @@ def get_molecule_from_file_fchk(file_name, read_multiple=False):
     key_list = ['Charge', 'Multiplicity', 'Number of alpha electrons', 'Number of beta electrons', 'Atomic numbers',
                 'Current cartesian coordinates', 'Shell type', 'Number of primitives per shell', 'Shell to atom map',
                 'Primitive exponents', 'Contraction coefficients', 'P(S=P) Contraction coefficients',
-                'Alpha Orbital Energies', 'Alpha MO coefficients', 'Beta MO coefficients']
+                'Alpha Orbital Energies', 'Beta Orbital Energies', 'Alpha MO coefficients', 'Beta MO coefficients']
     input_molecule = [[] for _ in range(len(key_list))]
     read = False
     with open(file_name, mode='r') as lines:
@@ -183,25 +235,31 @@ def get_molecule_from_file_fchk(file_name, read_multiple=False):
                             positions=coordinates,
                             name=name)
 
-        Ca = np.array(input_molecule[13], dtype=float).reshape(-1, int(np.sqrt(len(input_molecule[13]))))
-        energies_alpha = np.array(input_molecule[12], dtype=float).tolist()
-        if input_molecule[14]:
-            Cb = np.array(input_molecule[14], dtype=float).reshape(-1, int(np.sqrt(len(input_molecule[14]))))
+        alpha_energies = np.array(input_molecule[12], dtype=float).tolist()
+        if input_molecule[13]:
+            beta_energies = np.array(input_molecule[13], dtype=float).tolist()
+        else:
+            beta_energies = []
+
+        Ca = np.array(input_molecule[14], dtype=float).reshape(-1, int(np.sqrt(len(input_molecule[14]))))
+        if input_molecule[15]:
+            Cb = np.array(input_molecule[15], dtype=float).reshape(-1, int(np.sqrt(len(input_molecule[15]))))
         else:
             Cb = []
 
-        electronic_structure = ElectronicStructure(charge=input_molecule[0][0],
-                                                   multiplicity=input_molecule[1][0],
-                                                   basis=basis,
+        electronic_structure = ElectronicStructure(basis=basis,
                                                    orbital_coefficients=[Ca, Cb],
-                                                   mo_energies=energies_alpha,
-                                                   alpha_electrons=input_molecule[2],
-                                                   beta_electrons=input_molecule[3])
+                                                   # charge=input_molecule[0][0],
+                                                   multiplicity=input_molecule[1][0],
+                                                   alpha_energies=alpha_energies,
+                                                   beta_energies=beta_energies,
+                                                   alpha_occupancy=[1]*int(input_molecule[2][0]),
+                                                   beta_occupancy=[1]*int(input_molecule[3][0]))
 
         if read_multiple:
             return [Molecule(geometry, electronic_structure)]
-        else:
-            return Molecule(geometry, electronic_structure)
+
+        return Molecule(geometry, electronic_structure)
 
 
 def get_molecule_from_file_molden(file_name, read_multiple=False):
@@ -225,7 +283,7 @@ def get_molecule_from_file_molden(file_name, read_multiple=False):
     with open(file_name, mode='r') as lines:
         lines.readline()
         lines.readline()
-        name = lines.readline()
+        name = file_name.split('.')[0]
         for line in _non_blank_lines(lines):
 
             if '[' in line:
@@ -262,7 +320,7 @@ def get_molecule_from_file_molden(file_name, read_multiple=False):
                             spin = re.split('[= ]', line)[-1]
                             input_molecule[spin + ' MO coefficients'].append([])
                         elif 'Occup' in line:
-                            occupation[spin].append(float(line.split('=')[-1]))
+                            occupation[spin].append(int(float(line.split('=')[-1])))
                         else:
                             input_molecule[spin+' MO coefficients'][-1].append(line.split()[1])
 
@@ -290,21 +348,22 @@ def get_molecule_from_file_molden(file_name, read_multiple=False):
         symbols = []
         for atom_number in input_molecule['Atomic numbers']:
             total_n_electrons += int(atom_number)
-            symbols.append(tools.atomic_number_to_element(atom_number))
+            symbols.append(atomic_number_to_element(atom_number))
 
-        occupation['Alpha'] = np.array(occupation['Alpha'], dtype=float)
-        occupation['Beta'] = np.array(occupation['Beta'], dtype=float)
+        new_occupation = []
         if len(occupation['Beta']) == 0:
-            if 2.0 not in occupation['Alpha']:
-                occupation['Alpha'] = 2*occupation['Alpha']
-            n_electrons = sum(occupation['Alpha'])
-            input_molecule['Multiplicity'].append(sum(map(lambda x: x % 2 == 1, occupation['Alpha'])) + 1)
-        else:
-            if 2.0 not in occupation['Beta']:
-                occupation['Beta'] = 2*occupation['Beta']
-            n_electrons = sum(occupation['Alpha'] + occupation['Beta'])
-            input_molecule['Multiplicity'].append(sum(occupation['Alpha'] - occupation['Beta']) + 1)
-
+            if 2 not in occupation['Alpha']:
+                occupation['Beta'] = occupation['Alpha']
+            else:
+                for occup in occupation['Alpha']:
+                    new_occupation.append(occup//2)
+                    if occup == 2:
+                        occupation['Beta'].append(1)
+                    else:
+                        occupation['Beta'].append(0)
+                occupation['Alpha'] = new_occupation
+        n_electrons = sum(occupation['Alpha'] + occupation['Beta'])
+        input_molecule['Multiplicity'].append(sum(occupation['Alpha']) - sum(occupation['Beta']) + 1)
         input_molecule['Charge'].append(int(total_n_electrons - n_electrons))
 
         basis = basis_format(basis_set_name='UNKNOWN',
@@ -327,16 +386,20 @@ def get_molecule_from_file_molden(file_name, read_multiple=False):
         else:
             Cb = []
 
-        ee = ElectronicStructure(charge=input_molecule['Charge'][0],
-                                 multiplicity=input_molecule['Multiplicity'][0],
-                                 basis=basis,
+        warnings.warn('Beta energies not implemented yet')
+        ee = ElectronicStructure(basis=basis,
                                  orbital_coefficients=[Ca, Cb],
-                                 mo_energies=input_molecule['MO Energies'])
+                                 # charge=input_molecule['Charge'][0],
+                                 multiplicity=input_molecule['Multiplicity'][0],
+                                 alpha_energies=input_molecule['MO Energies'],
+                                 beta_energies=[],
+                                 alpha_occupancy=occupation['Alpha'],
+                                 beta_occupancy=occupation['Beta'])
 
         if read_multiple:
             return [Molecule(geometry, ee)]
-        else:
-            Molecule(geometry, ee)
+
+        return Molecule(geometry, ee)
 
 
 def get_geometry_from_file_ref(file_name, read_multiple=False):
@@ -353,8 +416,10 @@ def get_geometry_from_file_ref(file_name, read_multiple=False):
         lines.readline()
         name = lines.readline().split()[0]
         for line in lines:
-            if '$' in line or '#' in line:
+            if line.lstrip().startswith(comment_line):
                 pass
+            # elif line.strip() == '':
+            #     warnings.warn('Line {} is empty'.format(idl + 1), custom_errors.EmptyLineWarning)
             else:
                 try:
                     if len(line.split()) > 3:
@@ -366,7 +431,7 @@ def get_geometry_from_file_ref(file_name, read_multiple=False):
                 except (ValueError, IndexError):
                     if input_molecule:
                         structures.append(Geometry(positions=input_molecule,
-                                                  name=name))
+                                                   name=name))
                     input_molecule = []
                     name = line.split()[0]
         structures.append(Geometry(positions=input_molecule,
@@ -524,3 +589,132 @@ def reformat_input(array):
             else:
                 flat_list.append(item)
     return flat_list
+
+
+def get_array_txt(label, type, array, row_size=5):
+
+    formats = {'R': '15.8e',
+               'I': '11'}
+
+    n_elements = len(array)
+    rows = int(np.ceil(n_elements/row_size))
+
+    txt_fchk = '{:40}   {}   N=       {:5}\n'.format(label, type, n_elements)
+
+    for i in range(rows):
+        if (i+1)*row_size > n_elements:
+            txt_fchk += (' {:{fmt}}'* (n_elements - i*row_size) + '\n').format(*array[i * row_size:n_elements],
+                                                                               fmt=formats[type])
+        else:
+            txt_fchk += (' {:{fmt}}'* row_size  + '\n').format(*array[i * row_size: (i+1)*row_size],
+                                                               fmt=formats[type])
+
+    return txt_fchk
+
+
+def build_fchk(molecule):
+
+
+    structure = np.array(molecule.geometry.get_positions())
+    basis = molecule.electronic_structure.basis
+    alpha_mo_coeff = molecule.electronic_structure.coefficients_a
+    alpha_mo_energies = molecule.electronic_structure.alpha_energies
+    if molecule.electronic_structure.coefficients_b != molecule.electronic_structure.coefficients_a:
+        beta_mo_coeff = molecule.electronic_structure.coefficients_b
+        beta_mo_energies = molecule.electronic_structure.beta_energies
+        beta_mo_coeff = np.array(beta_mo_coeff).flatten().tolist()
+    else:
+        beta_mo_coeff = None
+        beta_mo_energies = None
+
+    number_of_basis_functions = len(alpha_mo_coeff)
+    alpha_electrons = molecule.electronic_structure.alpha_electrons
+    beta_electrons = molecule.electronic_structure.beta_electrons
+    number_of_electrons = alpha_electrons + beta_electrons
+
+    alpha_mo_coeff = np.array(alpha_mo_coeff).flatten().tolist()
+
+    atomic_numbers = [element_to_atomic_number(symbol) for symbol in molecule.geometry.get_symbols()]
+    total_energy = np.dot(alpha_mo_energies, alpha_electrons) + np.dot(beta_mo_energies, beta_electrons)
+
+    shell_type_list = {'s':  {'type':  0, 'angular_momentum': 0},
+                       'p':  {'type':  1, 'angular_momentum': 1},
+                       'd':  {'type':  2, 'angular_momentum': 2},
+                       'f':  {'type':  3, 'angular_momentum': 3},
+                       'sp': {'type': -1, 'angular_momentum': 1},  # hybrid
+                       'd_': {'type': -2, 'angular_momentum': 2},  # pure
+                       'f_': {'type': -3, 'angular_momentum': 3}}  # pure
+
+    shell_type = []
+    p_exponents = []
+    c_coefficients = []
+    p_c_coefficients = []
+    n_primitives = []
+    atom_map = []
+
+    largest_degree_of_contraction = 0
+    highest_angular_momentum = 0
+    number_of_contracted_shells = 0
+
+    for i, atoms in enumerate(basis['atoms']):
+        for shell in atoms['shells']:
+            number_of_contracted_shells += 1
+            st = shell['shell_type']
+            shell_type.append(shell_type_list[st]['type'])
+            n_primitives.append(len(shell['p_exponents']))
+            atom_map.append(i+1)
+            if highest_angular_momentum < shell_type_list[st]['angular_momentum']:
+                highest_angular_momentum = shell_type_list[st]['angular_momentum']
+
+            if len(shell['con_coefficients']) > largest_degree_of_contraction:
+                    largest_degree_of_contraction = len(shell['con_coefficients'])
+
+            for p in shell['p_exponents']:
+                p_exponents.append(p)
+            for c in shell['con_coefficients']:
+                c_coefficients.append(c)
+            for pc in shell['p_con_coefficients']:
+                p_c_coefficients.append(pc)
+
+    angstrom_to_bohr = 1/0.529177249
+    coordinates_list = angstrom_to_bohr*structure.flatten()
+
+    txt_fchk = '{}\n'.format('Extended Huckel calculation over filename')
+    txt_fchk += 'SP        RHuckel                                                     {}\n'.format('STO-3G')
+    txt_fchk += 'Number of atoms                            I               {}\n'.format(len(structure))
+    txt_fchk += 'Charge                                     I               {}\n'.format(molecule.get_charge())
+    txt_fchk += 'Multiplicity                               I               {}\n'.format(molecule.electronic_structure.
+                                                                                         multiplicity)
+    txt_fchk += 'Number of electrons                        I               {}\n'.format(number_of_electrons)
+    txt_fchk += 'Number of alpha electrons                  I               {}\n'.format(alpha_electrons)
+    txt_fchk += 'Number of beta electrons                   I               {}\n'.format(beta_electrons)
+    txt_fchk += 'Number of basis functions                  I               {}\n'.format(number_of_basis_functions)
+
+    txt_fchk += get_array_txt('Atomic numbers', 'I', atomic_numbers, row_size=6)
+    txt_fchk += get_array_txt('Nuclear charges', 'R', atomic_numbers)
+    txt_fchk += get_array_txt('Current cartesian coordinates', 'R', coordinates_list)
+
+    txt_fchk += 'Number of contracted shells                I               {}\n'.format(number_of_contracted_shells)
+    txt_fchk += 'Number of primitive shells                 I               {}\n'.format(np.sum(n_primitives))
+    txt_fchk += 'Highest angular momentum                   I               {}\n'.format(highest_angular_momentum)
+    txt_fchk += 'Largest degree of contraction              I               {}\n'.format(largest_degree_of_contraction)
+
+    txt_fchk += get_array_txt('Shell types', 'I', shell_type, row_size=6)
+    txt_fchk += get_array_txt('Number of primitives per shell', 'I', n_primitives, row_size=6)
+    txt_fchk += get_array_txt('Shell to atom map', 'I', atom_map, row_size=6)
+    txt_fchk += get_array_txt('Primitive exponents', 'R', p_exponents)
+    txt_fchk += get_array_txt('Contraction coefficients', 'R', c_coefficients)
+    txt_fchk += get_array_txt('P(S=P) Contraction coefficients', 'R', p_c_coefficients)
+    # txt_fchk += get_array_txt('Coordinates of each shell', 'R', coor_shell) #
+    # txt_fchk += get_array_txt('Overlap Matrix', 'R', overlap)
+    #txt_fchk += get_array_txt('Core Hamiltonian Matrix', 'R', core_hamiltonian)
+    txt_fchk += 'Total Energy                               R             {}\n'.format(total_energy)
+    txt_fchk += get_array_txt('Alpha Orbital Energies', 'R', alpha_mo_energies)
+    if beta_mo_energies is not None:
+        txt_fchk += get_array_txt('Beta Orbital Energies', 'R', beta_mo_energies)
+    # txt_fchk += get_array_txt('Total SCF Density', 'R', scf_density)
+    txt_fchk += get_array_txt('Alpha MO coefficients', 'R', alpha_mo_coeff)
+    if beta_mo_coeff is not None:
+        txt_fchk += get_array_txt('Beta MO coefficients', 'R', beta_mo_coeff)
+
+    return txt_fchk
