@@ -1,10 +1,11 @@
 import huckelpy
+from huckelpy import file_io
 
 
 class ExtendedHuckel:
 
-    def __init__(self, geometry):
-        self._EH = huckelpy.ExtendedHuckel(geometry.get_positions(), geometry.get_symbols())
+    def __init__(self, geometry, charge=0):
+        self._EH = huckelpy.ExtendedHuckel(geometry.get_positions(), geometry.get_symbols(), charge=charge)
         self._alpha_electrons = None
         self._beta_electrons = None
         self._total_electrons = self._EH.get_number_of_electrons()
@@ -30,5 +31,5 @@ class ExtendedHuckel:
         return self._total_electrons - self._alpha_electrons
 
     def build_fchk_file(self, name):
-        txt_fchk = huckelpy.file_io.build_fchk(self._EH)
+        txt_fchk = file_io.build_fchk(self._EH)
         open(name + '.fchk', 'w').write(txt_fchk)
