@@ -1,6 +1,7 @@
 from wfnsympy import WfnSympy
 from symgroupy import Symgroupy
-from cosymlib import tools
+from cosymlib.molecule.electronic_structure import ElectronicStructure
+from cosymlib.molecule.electronic_structure import ProtoElectronicStructure
 import numpy as np
 
 
@@ -25,6 +26,25 @@ def _get_key_wfnsym(group, vector_axis1, vector_axis2, center, alpha_occupancy,
 
 
 class Symmetry:
+    """
+    Symmetry main class
+
+    :param structure: a geometry, molecule or array type object
+    :type structure: Geometry, Molecule, np.array
+    :param central_atom: central atom position
+    :type central_atom: int
+    :param center: center of symmetry in Cartesian coordinates. By default center is optimized
+    :type center: list
+    :param connect_thresh: Connectivity threshold (Ionic radius is used as reference)
+    :type connect_thresh: float
+    :param multi: Number of symmetry axis to find
+    :type multi: int
+    :param axis: Main symmetry axis (If None, then optimize)
+    :type axis: list
+    :param axis2: secondary symmetry axis (If None, then optimize)
+    :type axis2: list
+    """
+
     def __init__(self,
                  structure,
                  central_atom=0,
@@ -35,23 +55,22 @@ class Symmetry:
                  axis2=None,
                  ):
 
-        # Allow geometry or molecule to be imported instead of crude Cartesian coordinates
         try:
-            # provided geometry
+            # Interpret as Geometry or molecule
             self._coordinates = structure.get_positions()
             self._symbols = structure.get_symbols()
             self._connectivity = structure.get_connectivity()
         except AttributeError:
-            try:
-                # provided molecule
-                self._coordinates = structure.geometry.get_positions()
-                self._symbols = structure.geometry.get_symbols()
-                self._connectivity = structure.geometry.get_connectivity()
-            except AttributeError:
-                # provided coordinates matrix
-                self._coordinates = structure
-                self._symbols = None
-                self._connectivity = None
+            # Interpret as numpy array of coordinates or list of lists
+            self._coordinates = structure
+            self._symbols = None
+            self._connectivity = None
+
+        # If molecule object add electronic structure
+        try:
+            self._electronic_structure = structure.electronic_structure
+        except AttributeError:
+            self._electronic_structure = None
 
         self._central_atom = central_atom
         self._center = center
@@ -61,13 +80,14 @@ class Symmetry:
         self._axis2 = axis2
         self._results = {}
 
-        try:
-            self._electronic_structure = structure.electronic_structure
-        except AttributeError:
-            self._electronic_structure = None
-
     # Modifier methods
     def set_parameters(self, parameters_dict):
+        """
+        Set symmetry calculation related parameters
+
+        :param parameters_dict: parameters in dictionary
+        :type parameters_dict: dict
+        """
         for name, value in parameters_dict.items():
             setattr(self, '_' + name, value)
 
@@ -87,7 +107,8 @@ class Symmetry:
         """
 
         # Crude calculation call methods
-        key = _get_key_symgroup(group, self._center, self._central_atom, self._connectivity, self._multi, self._connect_thresh)
+        key = _get_key_symgroup(group, self._center, self._central_atom, self._connectivity, self._multi,
+                                self._connect_thresh)
         if key not in self._results:
             self._results[key] = Symgroupy(self._coordinates,
                                            group=group,
@@ -102,22 +123,52 @@ class Symmetry:
         return self._results[key]
 
     def _get_wfnsym_results(self, group):
+        if self._electronic_structure is None:
+            raise Exception('Electronic structure not set')
+
+        elif isinstance(self._electronic_structure, ElectronicStructure):
+
+            key = _get_key_wfnsym(group, self._axis, self._axis2, self._center, self._electronic_structure.alpha_occupancy,
+                                  self._electronic_structure.beta_occupancy)
+
+            if key not in self._results:
+                self._results[key] = WfnSympy(coordinates=self._coordinates,
+                                              symbols=self._symbols,
+                                              basis=self._electronic_structure.basis,
+                                              center=self._center,
+                                              axis=self._axis,
+                                              axis2=self._axis2,
+                                              alpha_mo_coeff=self._electronic_structure.coefficients_a,
+                                              beta_mo_coeff=self._electronic_structure.coefficients_b,
+                                              group=group.upper(),
+                                              alpha_occupancy=self._electronic_structure.alpha_occupancy,
+                                              beta_occupancy=self._electronic_structure.beta_occupancy)
+        elif isinstance(self._electronic_structure, ProtoElectronicStructure):
+
+            key = _get_key_symgroup(group, self._center, self._central_atom, self._connectivity, self._multi,
+                                    self._connect_thresh)
+            if self._center is None:
+                coord_a = np.array(self._coordinates)
+                sym_m = []
+                for i in range(len(self._symbols)):
+                    sym_m.append(self._electronic_structure.basis['atoms'][i]['atomic_number'])
+                sym_m = np.array(sym_m)
+                self._center = [np.sum(coord_a[:, 0] * sym_m[:]) / np.sum(sym_m),
+                                              np.sum(coord_a[:, 1] * sym_m[:]) / np.sum(sym_m),
+                                              np.sum(coord_a[:, 2] * sym_m[:]) / np.sum(sym_m)]
 
-        key = _get_key_wfnsym(group, self._axis, self._axis2, self._center, self._electronic_structure.alpha_occupancy,
-                              self._electronic_structure.beta_occupancy)
 
-        if key not in self._results:
             self._results[key] = WfnSympy(coordinates=self._coordinates,
-                                          symbols=self._symbols,
-                                          basis=self._electronic_structure.basis,
-                                          center=self._center,
-                                          axis=self._axis,
-                                          axis2=self._axis2,
-                                          alpha_mo_coeff=self._electronic_structure.coefficients_a,
-                                          beta_mo_coeff=self._electronic_structure.coefficients_b,
-                                          group=group.upper(),
-                                          alpha_occupancy=self._electronic_structure.alpha_occupancy,
-                                          beta_occupancy=self._electronic_structure.beta_occupancy)
+                                              symbols=self._symbols,
+                                              basis=self._electronic_structure.basis,
+                                              alpha_mo_coeff=self._electronic_structure.coefficients_a,
+                                              group=group.upper(),
+                                              center=self._center,
+                                              axis=self._axis)
+
+        else:
+            raise ('Electronic structure class not recognized')
+
         return self._results[key]
 
     ##########################################
@@ -125,25 +176,86 @@ class Symmetry:
     ##########################################
 
     def measure(self, label):
+        """
+        Get symmetry measure
+
+        :param label: Point group label
+        :type label: str
+        :return: The measure
+        :rtype: float
+        """
         return self._get_symgroup_results(label).csm
 
     def nearest_structure(self, label):
+        """
+        Get nearest structure
+
+        :param label: Point group label
+        :type label: str
+        :return: The structure
+        :rtype: Structure
+        """
+        # TODO: Improve this docstring
+
         return self._get_symgroup_results(label).nearest_structure
 
     def optimum_axis(self, label):
+        """
+        Get the optimum main symmetry axis
+
+        :param label: Point group label
+        :type label: str
+        :return: The axis
+        :rtype: list
+        """
         return self._get_symgroup_results(label).optimum_axis
 
     def optimum_permutation(self, label):
+        """
+        Get the optimum atoms permutation
+
+        :param label: point group label
+        :return: The permutation
+        :rtype: list
+        """
         return self._get_symgroup_results(label).optimum_permutation
 
     def reference_axis(self, label):
+        """
+        Get reference axis
+
+        :param label: point group label
+        :type label: str
+        :return: The axis
+        :rtype: list
+        """
         return self._get_symgroup_results(label).reference_axis
 
-    def cms_multi(self, label, multi=1):
+    def csm_multi(self, label, multi=1):
+        """
+        Get symmetry measure of the optimum N axis
+
+        :param label: point group label
+        :type label: str
+        :param multi: number of axis
+        :type multi: int
+        :return: The measures
+        :rtype: list
+        """
         self._multi = multi
-        return self._get_symgroup_results(label).cms_multi
+        return self._get_symgroup_results(label).csm_multi
 
     def axis_multi(self, label, multi=1):
+        """
+        Get the optimum N axis
+
+        :param label: point group label
+        :type label: str
+        :param multi: number of axis
+        :type multi: int
+        :return: List of axis
+        :rtype: list
+        """
         self._multi = multi
         return self._get_symgroup_results(label).axis_multi
 
@@ -200,32 +312,27 @@ class Symmetry:
                 'table': results.ideal_gt}
 
     def dens_measure(self, group):
+
         results = self._get_wfnsym_results(group)
-        return {'labels': results.SymLab,
-                'csm': results.csm_dens,
-                'csm_coef': results.csm_dens_coef}
+        if isinstance(self._electronic_structure,ElectronicStructure):
+            return {'labels': results.SymLab,
+                    'csm': results.csm_dens,
+                    'csm_coef': results.csm_dens_coef,
+                    'self_similarity': results.self_similarity}
+        else:
+            return {'labels': results.SymLab,
+                    'csm': (1-results.mo_SOEVs_a[0].sum()/(results.mo_SOEVs_a[0][0]*len(results.mo_SOEVs_a[0]))),
+                    'csm_coef': results.mo_SOEVs_a[0] / results.mo_SOEVs_a[0][0],
+                    'self_similarity':  results.mo_SOEVs_a[0][0]}
+
 
     def axes(self, group):
         results = self._get_wfnsym_results(group)
+
         return {'center' : results.center,
                 'axis': results.axis,
                 'axis2': results.axis2}
 
-    # Old method to be deleted
-    def symmetry_overlap_analysis(self, group, vector_axis1, vector_axis2, center):
+    def symmetry_elements(self, group):
         results = self._get_wfnsym_results(group)
-
-        return [results.ideal_gt, results.SymLab, results.mo_SOEVs_a,
-                results.mo_SOEVs_b, results.wf_SOEVs_a, results.wf_SOEVs_b,
-                results.wf_SOEVs, results.grim_coef, results.csm_coef]
-
-    # To be deleted
-    def symmetry_irreducible_representation_analysis(self, group, vector_axis1, vector_axis2, center):
-        results = self._get_wfnsym_results(group)
-        return [results.IRLab, results.mo_IRd_a, results.mo_IRd_b,
-                results.wf_IRd_a, results.wf_IRd_b, results.wf_IRd]
-
-    # To be deleted
-    def old_symmetry_matrix(self, group, vector_axis1, vector_axis2, center):
-        results = self._get_wfnsym_results(group)
-        return results.SymMat
+        return {'SymAxes' : results.SymAxes}
